// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function [ c , y ] = condnb_condnum ( varargin )
  // Computes the empirical condition number of the function f at point x.
  //
  // Calling Sequence
  //   [ c , y ] = condnb_condnum ( f , x )
  //   [ c , y ] = condnb_condnum ( f , x , order )
  //   [ c , y ] = condnb_condnum ( f , x , order , h )
  //
  // Parameters
  //   f : a function with header y = f(x)
  //   x : a vector of doubles, the current point
  //   order : a 1x1 matrix of floating point integers, the order (Default order = 2). Available are order= 1, 2, 4, 6. If order==[], then the default order is used.
  //   h : a matrix of doubles, the step. Default tries to be optimal for accuracy. If h==[], then the default h is used.
  //   c : the condition number as the ratio ry/rx
  //   y : the computed value of f at point x
  //
  // Description
  //   Computes the condition number by using a finite difference formula.
  //
  //   Any optional input argument equal to the empty matrix is replaced by its default value.
  //
    //   The header of the function f must be
    //   <programlisting>
    //     y=f(x)
    //   </programlisting>
    //   where x is the current point and y is the function value.
    //
    // It might happen that the function requires additionnal arguments to be evaluated.
    // In this case, we can use the following feature.
    // The function f can also be the list (fun,a1,a2,...).
    // In this case fun, the first element in the list, must have the header:
    //   <programlisting>
    //     y = fun ( x , a1 , a2 , ... )
    //   </programlisting>
    // where the input arguments a1, a2, ...
    // will be automatically be appended at the end of the calling sequence.
  //
  // Examples
  //   // Show that sin can be ill-conditionned
  //   [ c , y ] = condnb_condnum ( sin , 1.e-100 )         // c~1
  //   // Compare with exact formula :
  //   condnb_sincond(1.e-100)
  //   [ c , y ] = condnb_condnum ( sin , 0 )               // c = nan
  //   [ c , y ] = condnb_condnum ( sin , 3.14159 )         // c ~10^6
  //   [ c , y ] = condnb_condnum ( sin , 3.141592653 )     // c ~10^9
  //   [ c , y ] = condnb_condnum ( sin , 3.1415926535898 ) // c ~10^14
  //   [ c , y ] = condnb_condnum ( sin , %pi )             // c ~10^16
  //   [ c , y ] = condnb_condnum ( sin , 1.000000357564167061e5 ) // c ~10^16
  //
  //   // An ill-conditionned case for the sum.
  //   xl = 10^(1:15);
  //   x = [-xl xl+0.1];
  //   yExpected = 1.5
  //   [ c , y ] = condnb_condnum ( sum , x )
  //   // Compare with exact formula :
  //   cExpected = condnb_sumcond(x) // c~10^15
  //
  //   // Check various finite difference orders for the sqrt function.
  //   // The condition number is 0.5.
  //   cc = condnb_condnum ( sqrt , 1 , 1 )
  //   // Compare with exact formula :
  //   cc = condnb_sqrtcond ( 1 )
  //   cc = condnb_condnum ( sqrt , 1 , 2 )
  //   cc = condnb_condnum ( sqrt , 1 , 4 )
  //   cc = condnb_condnum ( sqrt , 1 , 6 )
  //   // Make a loop
  //   x = 1.;
  //   order = [1 2 4 6];
  //   for k = 1:4
  //     cc = condnb_condnum ( sqrt , x , order(k) );
  //     ce = sqrtcond ( x );
  //     disp([order(k) cc ce])
  //   end
  //
  // // Use default order, but configure h
  // [cc,yc] = condnb_condnum ( sqrt , 1 , [] , 1.e-8 )
  // // Use default order and h
  // [cc,yc] = condnb_condnum ( sqrt , 1 , [] , [] )
  // // Set order, use default h
  // [cc,yc] = condnb_condnum ( sqrt , 1 , 4 , [] );
  //
  // Authors
  //   Michael Baudin, Digiteo, 2010
  //

  [lhs, rhs] = argn()
  apifun_checkrhs ( "condnb_condnum" , rhs , 2:4 )
  apifun_checklhs ( "condnb_condnum" , lhs , 0:2 )
  //
  // Get arguments
  __condnb_condnum_f = varargin ( 1 )
  x = varargin ( 2 )
  order = argindefault ( rhs , varargin , 3 , 2 )
  h = argindefault ( rhs , varargin , 4 , [] )
  hflag = ( h <> [] )
  //
  // Check types
  apifun_checktype ( "condnb_condnum" , __condnb_condnum_f , "f" , 1 , [ "function" "fptr" "list" ] )
  apifun_checktype ( "condnb_condnum" , x , "x" , 2 , "constant" )
  apifun_checktype ( "condnb_condnum" , order , "order" , 3 , "constant" )
  apifun_checktype ( "condnb_condnum" , h , "h" , 4 , "constant" )
  //
  // Check size
  apifun_checkvector ( "condnb_condnum" , x , "x" , 2 , size(x,"*") )
  apifun_checkscalar ( "condnb_condnum" , order , "order" , 3 )
  //
  // Check content
  apifun_checkoption ( "condnb_condnum" , order , "order" , 3 , [1 2 4 6] )
  //
  // Make x a column vector
  x = x(:)
  //
  // Compute the step
  if ( ~hflag ) then
    h = condnb_condnum_optimalh ( order ) 
    h = h * (1+1.d-3*abs(x))
    // Another strategy : h = max(h , h*abs(x) )
  end
  y = condnb_condnum_evalf ( __condnb_condnum_f , x )
  //
  // Check size of y
  if ( size(y,"r") <> 1 & size(y,"c")  <> 1 ) then
    errmsg = sprintf ( gettext ( "%s: Unexpected size of %s : variable has size %d while %d is expected.") , "condnb_condnum" , "y=f(x)" , size(y,"*") , 1 )
    error(errmsg)
  end
  //
  // Compute jacobian
  jac = condnb_condnum_firstder ( __condnb_condnum_f , x , order , h )
  // Make jac a column vector
  jac = jac(:)
  if ( norm(x) == 0 ) then
    // rx may be %inf or %nan, depending on norm(h)
    rx = norm(h) * %inf 
  else
    rx = norm(h) / norm(x)
  end
  if ( abs(y) == 0 ) then
    // ry may be %inf or %nan, depending on abs(jac' * h)
    ry = abs(jac' * h) * %inf
  else  
    ry = abs(jac' * h) / abs(y)
  end
  if ( rx == 0 ) then
    // c may be %inf or %nan, depending on ry
    c = ry * %inf
  else  
    c = ry / rx
  end
endfunction

function y = condnb_condnum_evalf ( __condnb_condnum_f , x )
  // Evaluates the function at point x.
  // The function can be a "function" or "fptr" : directly call it.
  // If the function is a list (type=15), the first element is the function, 
  // the remaining items are the arguments.
  if ( type(__condnb_condnum_f) == 15 ) then
    __condnb_condnum_f_f = __condnb_condnum_f ( 1 )
    y = __condnb_condnum_f_f ( x , __condnb_condnum_f(2:$) )
  else
    y = __condnb_condnum_f ( x )
  end
endfunction

function jac = condnb_condnum_firstder ( __condnb_condnum_f , x , order , h )
  // Computes the first derivative (Jacobian matrix) by a finite difference formula
  // http://reference.wolfram.com/mathematica/tutorial/NDSolvePDE.html#c:4 
  // (Reference for order = 6)
  if ( order == 1 ) then
    fx = condnb_condnum_evalf ( __condnb_condnum_f , x )
    fxp = condnb_condnum_evalf ( __condnb_condnum_f , x+h )
    jac = (fxp - fx)/h
  elseif ( order == 2 ) then
    fxp = condnb_condnum_evalf ( __condnb_condnum_f , x+h )
    fxm = condnb_condnum_evalf ( __condnb_condnum_f , x-h )
    jac = (fxp - fxm)/(2*h)
  elseif ( order == 4 ) then
    fxp = condnb_condnum_evalf ( __condnb_condnum_f , x+h )
    fxm = condnb_condnum_evalf ( __condnb_condnum_f , x-h )
    fxpp = condnb_condnum_evalf ( __condnb_condnum_f , x+2*h )
    fxmm = condnb_condnum_evalf ( __condnb_condnum_f , x-2*h )
    jac = (-fxpp + 8*fxp - 8*fxm + fxmm)/(12*h)
  elseif ( order == 6 ) then
    fxp = condnb_condnum_evalf ( __condnb_condnum_f , x+h )
    fxm = condnb_condnum_evalf ( __condnb_condnum_f , x-h )
    fxpp = condnb_condnum_evalf ( __condnb_condnum_f , x+2*h )
    fxmm = condnb_condnum_evalf ( __condnb_condnum_f , x-2*h )
    fxppp = condnb_condnum_evalf ( __condnb_condnum_f , x+3*h )
    fxmmm = condnb_condnum_evalf ( __condnb_condnum_f , x-3*h )
    jac = (fxppp - 9*fxpp + 45*fxp - 45*fxm + 9*fxmm - fxmmm)/(60*h)
  else
    errmsg = sprintf ( gettext ( "%s: Unexpected order = %d provided.") , "condnb_condnum_firstder" , order )
    error(errmsg)
  end
endfunction
function h = condnb_condnum_optimalh ( order )
  // Returns an optimal step h given order
  // alpha1 : the power of h in the bottom term in the finite difference formula : jac = (.*f(.)+.*f(.)+...)/(.*h^alpha1)
  if ( order == 1 ) then
    alpha1 = 1
  elseif ( order == 2 ) then
    alpha1 = 1
  elseif ( order == 4 ) then
    alpha1 = 1
  elseif ( order == 6 ) then
    alpha1 = 1
  else
    errmsg = sprintf ( gettext ( "%s: Unexpected order = %d provided.") , "condnb_condnum_optimalh" , order )
    error(errmsg)
  end
  h = %eps^(1/(alpha1+order))
endfunction
function argin = argindefault ( rhs , vararglist , ivar , default )
  // Returns the value of the input argument #ivar.
  // If this argument was not provided, or was equal to the 
  // empty matrix, returns the default value.
  if ( rhs < ivar ) then
    argin = default
  else
    if ( vararglist(ivar) <> [] ) then
      argin = vararglist(ivar)
    else
      argin = default
    end
  end
endfunction

